C   14/04/87 706021359  MEMBER NAME  DLB2M    *.FORT     M  FORTRAN77   0000
C******************** DLB2M ******************************************* 0000
      SUBROUTINE DLB2M (A,N,NLD,NA,B,EPS,IOPT,WK,IER)                   0000
C                                                                       0000
C-----------------------------------------------------------------------0000
C                                                                       0000
C   S-1511-1 COPYRIGHT HITACHI,LTD. 1980                                0000
C                                                                       0000
C   NAME                - DLB2M : DOUBLE PRECISION                      0000
C                                                                       0000
C   USAGE               - CALL DLB2M (A,N,NLD,NA,B,EPS,IOPT,WK,IER)     0000
C                                                                       0000
C   FUNCTION            - BY THE MODIFIED CHOLESKY METHOD, WE SOLVE     0000
C                           THE SYSTEM OF LINEAR EQUATIONS IN N UN-     0000
C                           KNOWNS WITH A REAL SYMMETRIC BAND MATRIX.   0000
C                                                                       0000
C   ARGUMENTS  A(NA,N)  - INPUT. REAL SYMMETRIC BAND MATRIX.            0000
C                         OUTPUT. LOWER TRIANGULAR PARTS OF THE MATRIX  0000
C                           WHOSE MODIFIED CHOLESKY DECOMPOSITION HAS   0000
C                           BEEN COMPLETED.                             0000
C              N        - INPUT. NUMBER OF UNKNOWNS N (1<=N).           0000
C              NLD      - INPUT. BAND WIDTH OF THE MATRIX (0<=NLD<=N-1).0000
C              NA       - INPUT. NUMBER OF ROWS OF A IN THE DIMENSION   0000
C                           STATEMENT OF A MAIN PROGRAM (NA>=NLD+1).    0000
C              B(N)     - INPUT. RIGHT SIDE VECTOR.                     0000
C                         OUTPUT. SOLUTIONS OF THE EQUATION.            0000
C              EPS      - INPUT. TOLERANCE FOR THE SINGULARITY (EPS>=   0000
C                           0.0). WHEN EPS<0.0, STANDARD VALUE IS       0000
C                           ASSUMED.                                    0000
C              IOPT     - INPUT.                                        0000
C                           IF IOPT=1, BOTH DECOMPOSITION AND THE       0000
C                              SOLUTION OF THE EQUATION ARE OBTAINED.   0000
C                           IF IOPT=2, ONLY DECOMPOSITION IS OBTAINED.  0000
C                           IF IOPT=3, SOLUTION OF THE EQUATION IS      0000
C                              OBTAINED. WE SPECIFY IT AFTER CARRYING   0000
C                              OUT THE SUBROUTINE WITH IOPT=1,2.        0000
C              WK(N)    - WORK AREA.                                    0000
C              IER      - ERROR INDICATOR.                              0000
C                           IER=   0, NO ERROR WAS DETECTED.            0000
C                           IER=1000, THE MATRIX WAS NOT POSITIVE       0000
C                                     DEFINITE DECOMPOSITION CONTINUED. 0000
C                           IER=2000, N<1, NLD<0, NLD>N-1, NLD+1>NA,    0000
C                                     IOPT<1 OR IOPT>3.                 0000
C                           IER=3000, THE MATRIX IS NEARLY SINGULAR.    0000
C                                                                       0000
C   NOTICE              - WHEN IOPT=3, WE MUST SPECIFY IT AFTER CARRY-  0000
C                           ING OUT THE SUBROUTINE WITH IOPT=1,2 AND    0000
C                           MUST NOT CHANGE THE CONTENTS OF THE ARGU-   0000
C                           MENTS EXCEPT IOPT AND B.                    0000
C                                                                       0000
C   STATUS              - S-1511-1 04-00                                0000
C                                                                       0000
C   HISTORY             - DATE.    1980. 3                              0000
C                                  1980.11                              0000
C                                  1982. 4                              0000
C                                                                       0000
C-----------------------------------------------------------------------0000
C                                                                       0000
      IMPLICIT REAL*8 (A-H,O-Z)                                         0000
C     GENERIC                                                           0000
      DIMENSION A(NA,N),B(N),WK(N)                                      0000
      DATA ZERO,ONE,SIXTN      / 0.0D0,1.0D0,16.0D0                  /  0000
      DATA UEPS          / X'3410000000000000'/                         0000
C                                                                       0000
C     PARAMETER CHECK AND INITIALIZATION                                0000
C                                                                       0000
      IF (N.LT.1 .OR. NLD+1.GT.NA) GO TO 150                            0000
      IF (NLD.LT.0 .OR. NLD.GT.N-1) GO TO 150                           0000
      IF (IOPT.LT.1 .OR. IOPT.GT.3) GO TO 150                           0000
      M=NLD+1                                                           0000
      IF (IOPT .EQ. 3) GO TO 80                                         0000
      SEPS=EPS                                                          0000
      IF (EPS .LT. ZERO) SEPS=FLOAT(NLD)*SIXTN*UEPS                     0000
      IER=0                                                             0000
C                                                                       0000
C     MODIFIED CHOLESKY DECOMPOSITION                                   0000
C                                                                       0000
      DO 70 K=1,N                                                       0000
         JBS=MAX(1,M-K+1)                                               0000
         IF (JBS+1 .GT. NLD) GO TO 30                                   0000
         JBS1=JBS+1                                                     0000
         DO 20 JB=JBS1,NLD                                              0000
            I=K-M+JB                                                    0000
            JJBE=JB-1                                                   0000
            MJB=M-JB                                                    0000
            SUM=ZERO                                                    0000
            DO 10 JJB=JBS,JJBE                                          0000
               SUM=SUM+A(JJB,K)*A(JJB+MJB,I)                            0000
   10       CONTINUE                                                    0000
            A(JB,K)=A(JB,K)-SUM                                         0000
   20    CONTINUE                                                       0000
   30    SUM=ZERO                                                       0000
         IF (JBS .GE. M) GO TO 50                                       0000
         KM=K-M                                                         0000
         DO 40 JJB=JBS,NLD                                              0000
            DT=A(JJB,K)*WK(JJB+KM)                                      0000
            SUM=SUM+DT*A(JJB,K)                                         0000
            A(JJB,K)=DT                                                 0000
   40    CONTINUE                                                       0000
   50    T=A(M,K)-SUM                                                   0001
         IF (ABS(T) .LE. ABS(SUM)*SEPS) GO TO 160                       0001
         IF (T .GT. ZERO) GO TO 60                                      0001
      IER=1000                                                          0001
      WRITE(6,55) IER                                                   0001
   55 FORMAT(' *** SUBR.DLB2M *** WARNING  NON-POSITIVE DEFINITE. ',    0001
     %   'IER=',I4)                                                     0001
   60    A(M,K)=ONE/T                                                   0001
         WK(K)=A(M,K)                                                   0001
   70 CONTINUE                                                          0001
      IF (IOPT .EQ. 2) GO TO 300                                        0001
C                                                                       0001
C     FORWARD SUBSTITUTION                                              0001
C                                                                       0001
   80 IF (IER .GT. 2000) GO TO 170                                      0001
      IF (N.EQ.1 .OR. NLD.EQ.0) GO TO 110                               0001
      DO 100 I=2,N                                                      0001
         JBS=MAX(1,M-I+1)                                               0001
         IM=I-M                                                         0001
         SUM=ZERO                                                       0001
         DO 90 JB=JBS,NLD                                               0001
            SUM=SUM+A(JB,I)*B(JB+IM)                                    0001
   90    CONTINUE                                                       0001
         B(I)=B(I)-SUM                                                  0001
  100 CONTINUE                                                          0001
  110 DO 120 I=1,N                                                      0001
         B(I)=B(I)*WK(I)                                                0001
  120 CONTINUE                                                          0001
C                                                                       0001
C     BACKWARD REDUCTION                                                0001
C                                                                       0001
      IF (N.EQ.1 .OR. NLD.EQ.0) GO TO 300                               0001
      DO 140 K=2,N                                                      0001
         J=N-K+2                                                        0001
         IF (B(J) .EQ. ZERO) GO TO 140                                  0001
            IBS=MAX(1,M-J+1)                                            0001
            T=-B(J)                                                     0001
            JM=J-M                                                      0001
            DO 130 IB=IBS,NLD                                           0001
               B(IB+JM)=B(IB+JM)+T*A(IB,J)                              0001
  130       CONTINUE                                                    0001
  140 CONTINUE                                                          0001
C                                                                       0001
C     NORMAL RETURN                                                     0001
C                                                                       0001
      GO TO 300                                                         0001
C                                                                       0001
C     PARAMETER ERROR                                                   0001
C                                                                       0001
  150 IER=2000                                                          0001
      WRITE(6,155) IER                                                  0001
  155 FORMAT(' *** SUBR.DLB2M *** PARAMETER ERROR. IER=',I4)            0001
      GO TO 300                                                         0001
C                                                                       0001
C     MATRIX IS SINGULAR                                                0001
C                                                                       0001
  160 IER=3000                                                          0001
      WRITE(6,165) IER                                                  0001
  165 FORMAT(' *** SUBR.DLB2M *** MATRIX SINGULAR. IER=',I4)            0001
      GO TO 300                                                         0001
C                                                                       0001
C     INVALID IER                                                       0001
C                                                                       0001
  170 WRITE(6,180)                                                      0001
  180 FORMAT(' *** SUBR.DLB2M *** ERROR IN PREVIOUS CALL FOR ',         0001
     %  'DECOMPOSITION.')                                               0001
  300 RETURN                                                            0001
      END                                                               0001
