C   24/06/87 709012011  MEMBER NAME  POISSN   *.FORT        FORTRAN77   
      SUBROUTINE POISSN(NX,NY,Dum1,dum2,IXM,IYM,Q,PHIQ,PHI,AAA,AAA2,W)   
C  SOLVE 2-DIM POISSON EQUATION                                         
C   - LAPLACIAN PHI(X,Y)= 2*PI*RHO(X,Y)                                 
C  BY BAND MATRIX. MORE SPECIFICALLY,                                   
C   (2*PHI(I,J)-PHI(I+1,J)-PHI(I-1,J))/DX**2                            
C   +(2*PHI(I,J)-PHI(I,J+1)-PHI(I,J-1))/DY**2                           
C   = 2*PI*Q(I,J)/(DX*DY)                                               
C                     (1<=I<=NX, 1<=J<=NY)                              
C  BOUNDARY CONDITION                                                   
C    PHI ON OUTERMOST CELLS ARE CALCULATED BY DIRECT COULOMB'S LAW.     
C  SUBR.POTMAT MUST BE CALLED IN ADVANCE TO PREPARE PHIQ,AAA AND AAA2.  
C                                                                       
      IMPLICIT REAL*8 (A-H,O-Z)                                         
      INTEGER IXM(2,2),IYM(2,2)                                         
      REAL*8 PHIQ(0:NX-1,0:NY-1),AAA(NX-1,(NX-2)*(NY-2)),               
     %       W(2:NX-1,2:NY-1),AAA2((NX-2)*(NY-2)),                      
     %   Q(NX,NY,2),PHI(NX,NY,2)                                        
      DATA PI/3.14159 26535 8979D0/                                     
**
*  94/11/21  by T.Takahashi
* add to much type of arguments
**
      real*4 dum1,dum2
      dx = dum1
      dy = dum2
C                                                                       
c      write(6,*) 'called poissn',dx,dy
      NDIM=(NX-2)*(NY-2)                                                
      DO 960 L=1,2                                                      
C  POTENTIAL ON THE OUTER BOUNDARY                                      
      DO 820 J=1,NY                                                     
      PHI1=0D0                                                          
      PHI2=0D0                                                          
      DO 810 I1=IXM(1,L),IXM(2,L)                                       
      IX1=ABS(1-I1)                                                     
      IX2=ABS(NX-I1)                                                    
      DO 810 J1=IYM(1,L),IYM(2,L)                                       
      IY=ABS(J-J1)                                                      
      PHI1=PHI1+Q(I1,J1,L)*PHIQ(IX1,IY)                                 
  810 PHI2=PHI2+Q(I1,J1,L)*PHIQ(IX2,IY)                                 
      PHI(1,J,L)=PHI1                                                   
      PHI(NX,J,L)=PHI2                                                  
  820 CONTINUE                                                          
      DO 840 I=2,NX-1                                                   
      PHI1=0D0                                                          
      PHI2=0D0                                                          
      DO 830 J1=IYM(1,L),IYM(2,L)                                       
      IY1=ABS(1-J1)                                                     
      IY2=ABS(NY-J1)                                                    
      DO 830 I1=IXM(1,L),IXM(2,L)                                       
      IX=ABS(I-I1)                                                      
      PHI1=PHI1+Q(I1,J1,L)*PHIQ(IX,IY1)                                 
  830 PHI2=PHI2+Q(I1,J1,L)*PHIQ(IX,IY2)                                 
      PHI(I,1,L)=PHI1                                                   
      PHI(I,NY,L)=PHI2                                                  
  840 CONTINUE                                                          
C  POISSON EQUATION                                                     
      UDFL=1D60                                                         
      FAC1=DY/DX*UDFL                                                   
      FAC2=DX/DY*UDFL                                                   
      FAC4=2.*PI*UDFL                                                   
      DO 860 J=2,NY-1                                                   
      DO 860 I=2,NX-1                                                   
  860 W(I,J)=Q(I,J,L)*FAC4                                              
      DO 880 I=2,NX-1                                                   
      W(I,2)=W(I,2)+FAC2*PHI(I,1,L)                                     
  880 W(I,NY-1)=W(I,NY-1)+FAC2*PHI(I,NY,L)                              
      DO 900 J=2,NY-1                                                   
      W(2,J)=W(2,J)+FAC1*PHI(1,J,L)                                     
  900 W(NX-1,J)=W(NX-1,J)+FAC1*PHI(NX,J,L)                              
C         WRITE(6,9999) ((W(I,J),I=2,NX-1),J=NY-1,2,-1)                 
C9999     FORMAT(' ***W=',/,(1X,1P14D9.2))                              
      EPS=1D-12                                                         
      IOPTN=3                                                           
      IER=0                                                             
      CALL DLB2M(AAA,NDIM,NX-2,NX-1,W,EPS,IOPTN,AAA2,IER)               
C         WRITE(6,9999) ((W(I,J),I=2,NX-1),J=NY-1,2,-1)                 
      DO 920 J=2,NY-1                                                   
      DO 920 I=2,NX-1                                                   
  920 PHI(I,J,L)=W(I,J)                                                 
  960 CONTINUE                                                          
      RETURN                                                            
      END                                                               
